@@ -4,11 +4,9 @@ import { tests } from "./__fixtures__/tests";
 
 describe("Stringify & re-parse", () => {
     describe("Own tests", () => {
-        for (const [selector, expected, message, options] of tests) {
+        for (const [selector, expected, message] of tests) {
             test(`${message} (${selector})`, () => {
-                expect(parse(stringify(expected), options)).toStrictEqual(
-                    expected
-                );
+                expect(parse(stringify(expected))).toStrictEqual(expected);
             });
         }
     });
@@ -18,7 +16,7 @@ describe("Stringify & re-parse", () => {
             readFileSync(`${__dirname}/__fixtures__/out.json`, "utf8")
         );
         for (const s of Object.keys(out)) {
-            expect(parse(s)).toStrictEqual(out[s]);
+            expect(parse(stringify(out[s]))).toStrictEqual(out[s]);
         }
     });
 });
